<?php

namespace Drupal\hashicorp_vault_secrets;

use GuzzleHttp\ClientInterface;
use Drupal\Core\Logger\LoggerChannelInterface;

/**
 * Service to interact with hashiCorp Cloud API.
 */
class HashicorpApiService {

  /**
   * HTTP client.
   *
   * @var ClientInterface
   */
  protected $httpClient;

  /**
   * Logger channel.
   *
   * @var LoggerChannelInterface
   */
  protected $logger;


  /**
   * Constructor.
   *
   * @param \GuzzleHttp\ClientInterface $httpClient
   *   The Guzzle HTTP client.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger factory.
   */
  public function __construct(ClientInterface $httpClient, LoggerChannelInterface $logger) {
    $this->httpClient = $httpClient;
    $this->logger = $logger;
  }

  /**
   * Return Key value from hashicorp.
   *
   * @param string $organization_id
   *   The organization ID.
   * @param string $project_id
   *   The project ID.
   * @param string $app_name
   *   The app name.
   * @param string $Key_name
   *   The key name.
   * @param string $client_id
   *   The client ID.
   * @param string $client_secret
   *   The client secret.
   *
   * @return string
   *   The secret key value.
   */
  public function getKeyValue(string $organization_id, string $project_id, string $app_name, string $Key_name, string $client_id, string $client_secret): string {
    $access_token = $this->getAccessToken($client_id, $client_secret);
    if (empty($access_token)) {
      $this->logger->error('Please verify that the client ID and client secret are correct.');
      return '';
    }

    $secret_keys = $this->getSecrets($access_token, $organization_id, $project_id, $app_name);
    $secret_keys = $secret_keys['secrets'];
    foreach ($secret_keys as $key) {
      if ($key['name'] == $Key_name) {
        return $key['version']['value'];
      }
    }

    return '';
  }

  /**
   * Fetch the HCP API Token.
   *
   * @return string|null
   *   The access token, or null if the request fails.
   */
  private function getAccessToken($client_id, $client_secret): ?string {
    try {
      $response = $this->httpClient->post('https://auth.idp.hashicorp.com/oauth2/token', [
        'form_params' => [
          'client_id' => $client_id,
          'client_secret' => $client_secret,
          'grant_type' => 'client_credentials',
          'audience' => 'https://api.hashicorp.cloud',
        ],
        'headers' => [
          'Content-Type' => 'application/x-www-form-urlencoded',
        ],
      ]);

      $data = json_decode($response->getBody()->getContents(), TRUE);
      return $data['access_token'] ?? NULL;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to fetch access token: @message', ['@message' => $e->getMessage()]);
      return NULL;
    }
  }

  /**
   * Fetch secrets from hashiCorp cloud API using the access token.
   *
   * @param string $accessToken
   *   The access token.
   * @param string $organization_id
   *   The organization ID.
   * @param string $project_id
   *   The project ID.
   * @param string $app_name
   *   The app name.
   *
   * @return array|null
   *   The secrets data, or null if the request fails.
   */
  private function getSecrets(string $accessToken, string $organization_id, string $project_id, string $app_name): ?array {
    $url = 'https://api.cloud.hashicorp.com/secrets/2023-06-13/organizations/' . $organization_id . '/projects/' . $project_id . '/apps/' . $app_name . '/open';

    try {
      $response = $this->httpClient->get($url, [
        'headers' => [
          'Authorization' => 'Bearer ' . $accessToken,
        ],
      ]);

      $data = json_decode($response->getBody()->getContents(), TRUE);
      return $data;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to fetch secrets: @message', ['@message' => $e->getMessage()]);
      return NULL;
    }
  }

}
