<?php

namespace Drupal\hashicorp_vault_secrets\Plugin\KeyProvider;

use Drupal\key\Plugin\KeyProviderBase;
use Drupal\key\Entity\Key;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyInterface;
use Drupal\key\Plugin\KeyPluginFormInterface;
use Drupal\key\Plugin\KeyProviderSettableValueInterface;
use Drupal\hashicorp_vault_secrets\HashicorpApiService;
use Drupal\Core\Logger\LoggerChannelInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Retrieves the keys from hashiCorp vault.
 *
 * @KeyProvider(
 *   id = "hashicorp_vault_provider",
 *   label = "HashiCorp Vault Provider",
 *   description = @Translation("Retrieves the keys from hashiCorp vault."),
 *   key_value = {
 *     "accepted" = TRUE,
 *     "required" = TRUE
 *   }
 * )
 */
class HashicorpKeyProvider extends KeyProviderBase implements KeyProviderSettableValueInterface, KeyPluginFormInterface {

  /**
   * Logger channel.
   *
   * @var LoggerChannelInterface
   */
  protected $logger;

  /**
   * Hashicorp API service.
   *
   * @var HashicorpApiService
   */
  protected $hashicorp_api_service;

  /**
   * Constructs a new hashicorp_vault_provider.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   * @param LoggerChannelInterface $logger
   *   The hashicorp_vault_secrets Drupal logger channel.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    LoggerChannelInterface $logger,
    HashicorpApiService $hashicorp_api_service
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->logger = $logger;
    $this->hashicorp_api_service = $hashicorp_api_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.channel.hashicorp_vault_secrets'),
      $container->get('hashicorp_vault_secrets.hashicorp_api_service'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state
  ) {
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(
    array &$form,
    FormStateInterface $form_state
  ) {
  }

    /**
   * {@inheritdoc}
   */
  public function setKeyValue(KeyInterface $key, $key_value) {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteKeyValue(KeyInterface $key) {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(
    array &$form,
    FormStateInterface $form_state
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public function getKeyValue(KeyInterface $key) {
    // Retrieve the KeyType plugin from the key entity.
    $key_type = $key->getKeyType();

    if (!$key_type) {
      $this->logger->error('Key type not found.');
      return FALSE;
    }

    // Access the configuration from the KeyType plugin.
    $configuration = $key_type->getConfiguration();
    $organization_id = $configuration['organization_id'];
    $project_id = $configuration['project_id'];
    $app_name = $configuration['app_name'];
    $key_name = $configuration['key_name'];

    $client_id = Key::load($configuration['client_id'])->getKeyValue();
    $client_secret = Key::load($configuration['client_secret'])->getKeyValue();

    return $this->hashicorp_api_service->getKeyValue($organization_id, $project_id, $app_name, $key_name, $client_id, $client_secret);
  }
}
