<?php

namespace Drupal\hashicorp_vault_secrets\Plugin\KeyType;

use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\key\Plugin\KeyPluginFormInterface;
use Drupal\key\Plugin\KeyTypeBase;
use Drupal\key\Entity\Key;

/**
 * Defines a key type for encryption that generates keys with hashicorp_vault_secrets.
 *
 * @KeyType(
 *   id = "hashicorp_vault_secrets_encryption",
 *   label = @Translation("Hashicorp Vault Secrets Encryption"),
 *   description = @Translation("A key type used for encryption, Retrieving keys using hashicorp vault secrets."),
 *   group = "encryption",
 *   key_value = {
 *     "plugin" = "generate"
 *   }
 * )
 */
class HashicorpEncryptionKeyType extends KeyTypeBase
  implements KeyPluginFormInterface {

  /**
   * Constructs a new hashicorp_vault_secrets_encryption.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'organization_id' => '',
      'project_id' => '',
      'app_name' => '',
      'key_name' => '',
      'client_id' => '',
      'client_secret' => '',
      'key_size' => 256,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state
  ) {
    $config = $this->getConfiguration();
    $key_size_options = [
      '128' => 128,
      '192' => 192,
      '256' => 256,
    ];

    $key_size = $config['key_size'];

    $form['organization_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Hashicorp organization ID'),
      '#description' => $this->t('The hashicorp organization ID.'),
      '#required' => TRUE,
      '#default_value' => $config['organization_id'],
    ];

    $form['project_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Hashicorp project ID'),
      '#description' => $this->t('The hashicorp organization ID.'),
      '#required' => TRUE,
      '#default_value' => $config['project_id'],
    ];

    $form['app_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Hashicorp APP name'),
      '#description' => $this->t('The hashicorp APP name.'),
      '#required' => TRUE,
      '#default_value' => $config['app_name'],
    ];

    $form['key_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Hashicorp key name'),
      '#description' => $this->t('The hashicorp key name.'),
      '#required' => TRUE,
      '#default_value' => $config['key_name'],
    ];

    $form['client_id'] = [
      '#type' => 'key_select',
      '#title' => $this->t('Hashicorp key client ID'),
      '#description' => $this->t('The hashicorp client ID related to the key.'),
      '#default_value' => $config['client_id'],
      '#required' => TRUE,
      '#key_filters' => ['key_type' => 'authentication'],
    ];

    $form['client_secret'] = [
      '#type' => 'key_select',
      '#title' => $this->t('Hashicorp client secret'),
      '#description' => $this->t('The hashicorp client secret related to the key.'),
      '#default_value' => $config['client_secret'],
      '#required' => TRUE,
      '#key_filters' => ['key_type' => 'authentication'],
    ];

    $form['key_size'] = [
      '#type' => 'select',
      '#title' => $this->t('Key size'),
      '#description' => $this->t('The size of the key in bits.'),
      '#options' => $key_size_options,
      '#default_value' => $key_size,
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    // Default validation is fine.
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $this->configuration = $form_state->getValues();
  }

  /**
   * {@inheritdoc}
   */
  public static function generateKeyValue(array $configuration) {
    $organization_id = $configuration['organization_id'];
    $project_id = $configuration['project_id'];
    $app_name = $configuration['app_name'];
    $key_name = $configuration['key_name'];
    $client_id = Key::load($configuration['client_id']);
    $client_id = $client_id->getKeyValue();
    $client_secret = Key::load($configuration['client_secret']);
    $client_secret = $client_secret->getKeyValue();
    $hashicorp_api_service = \Drupal::service('hashicorp_vault_secrets.hashicorp_api_service');

    $key = $hashicorp_api_service->getKeyValue($organization_id, $project_id, $app_name, $key_name, $client_id, $client_secret);

    return $key;
  }

  /**
   * {@inheritdoc}
   */
  public function validateKeyValue(array $form, FormStateInterface $form_state, $key_value) {
    if (!$form_state->getValue('key_size')) {
      return;
    }

    // Validate the key size.
    $bytes = $form_state->getValue('key_size') / 8;
    if (strlen($key_value) != $bytes) {
      $form_state->setErrorByName('key_size', $this->t('The selected key size does not match the actual size of the key.'));
    }
  }

}

